"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright 2022 Google LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.type.expr_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Policy(google.protobuf.message.Message):
    """An Identity and Access Management (IAM) policy, which specifies access
    controls for Google Cloud resources.


    A `Policy` is a collection of `bindings`. A `binding` binds one or more
    `members`, or principals, to a single `role`. Principals can be user
    accounts, service accounts, Google groups, and domains (such as G Suite). A
    `role` is a named list of permissions; each `role` can be an IAM predefined
    role or a user-created custom role.

    For some types of Google Cloud resources, a `binding` can also specify a
    `condition`, which is a logical expression that allows access to a resource
    only if the expression evaluates to `true`. A condition can add constraints
    based on attributes of the request, the resource, or both. To learn which
    resources support conditions in their IAM policies, see the
    [IAM documentation](https://cloud.google.com/iam/help/conditions/resource-policies).

    **JSON example:**

        {
          "bindings": [
            {
              "role": "roles/resourcemanager.organizationAdmin",
              "members": [
                "user:mike@example.com",
                "group:admins@example.com",
                "domain:google.com",
                "serviceAccount:my-project-id@appspot.gserviceaccount.com"
              ]
            },
            {
              "role": "roles/resourcemanager.organizationViewer",
              "members": [
                "user:eve@example.com"
              ],
              "condition": {
                "title": "expirable access",
                "description": "Does not grant access after Sep 2020",
                "expression": "request.time < timestamp('2020-10-01T00:00:00.000Z')",
              }
            }
          ],
          "etag": "BwWWja0YfJA=",
          "version": 3
        }

    **YAML example:**

        bindings:
        - members:
          - user:mike@example.com
          - group:admins@example.com
          - domain:google.com
          - serviceAccount:my-project-id@appspot.gserviceaccount.com
          role: roles/resourcemanager.organizationAdmin
        - members:
          - user:eve@example.com
          role: roles/resourcemanager.organizationViewer
          condition:
            title: expirable access
            description: Does not grant access after Sep 2020
            expression: request.time < timestamp('2020-10-01T00:00:00.000Z')
        etag: BwWWja0YfJA=
        version: 3

    For a description of IAM and its features, see the
    [IAM documentation](https://cloud.google.com/iam/docs/).
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VERSION_FIELD_NUMBER: builtins.int
    BINDINGS_FIELD_NUMBER: builtins.int
    AUDIT_CONFIGS_FIELD_NUMBER: builtins.int
    ETAG_FIELD_NUMBER: builtins.int
    version: builtins.int
    """Specifies the format of the policy.

    Valid values are `0`, `1`, and `3`. Requests that specify an invalid value
    are rejected.

    Any operation that affects conditional role bindings must specify version
    `3`. This requirement applies to the following operations:

    * Getting a policy that includes a conditional role binding
    * Adding a conditional role binding to a policy
    * Changing a conditional role binding in a policy
    * Removing any role binding, with or without a condition, from a policy
      that includes conditions

    **Important:** If you use IAM Conditions, you must include the `etag` field
    whenever you call `setIamPolicy`. If you omit this field, then IAM allows
    you to overwrite a version `3` policy with a version `1` policy, and all of
    the conditions in the version `3` policy are lost.

    If a policy does not include any conditions, operations on that policy may
    specify any valid version or leave the field unset.

    To learn which resources support conditions in their IAM policies, see the
    [IAM documentation](https://cloud.google.com/iam/help/conditions/resource-policies).
    """
    etag: builtins.bytes
    """`etag` is used for optimistic concurrency control as a way to help
    prevent simultaneous updates of a policy from overwriting each other.
    It is strongly suggested that systems make use of the `etag` in the
    read-modify-write cycle to perform policy updates in order to avoid race
    conditions: An `etag` is returned in the response to `getIamPolicy`, and
    systems are expected to put that etag in the request to `setIamPolicy` to
    ensure that their change will be applied to the same version of the policy.

    **Important:** If you use IAM Conditions, you must include the `etag` field
    whenever you call `setIamPolicy`. If you omit this field, then IAM allows
    you to overwrite a version `3` policy with a version `1` policy, and all of
    the conditions in the version `3` policy are lost.
    """
    @property
    def bindings(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Binding]:
        """Associates a list of `members`, or principals, with a `role`. Optionally,
        may specify a `condition` that determines how and when the `bindings` are
        applied. Each of the `bindings` must contain at least one principal.

        The `bindings` in a `Policy` can refer to up to 1,500 principals; up to 250
        of these principals can be Google groups. Each occurrence of a principal
        counts towards these limits. For example, if the `bindings` grant 50
        different roles to `user:alice@example.com`, and not to any other
        principal, then you can add another 1,450 principals to the `bindings` in
        the `Policy`.
        """

    @property
    def audit_configs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___AuditConfig]:
        """Specifies cloud audit logging configuration for this policy."""

    def __init__(
        self,
        *,
        version: builtins.int = ...,
        bindings: collections.abc.Iterable[global___Binding] | None = ...,
        audit_configs: collections.abc.Iterable[global___AuditConfig] | None = ...,
        etag: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["audit_configs", b"audit_configs", "bindings", b"bindings", "etag", b"etag", "version", b"version"]) -> None: ...

global___Policy = Policy

@typing.final
class Binding(google.protobuf.message.Message):
    """Associates `members`, or principals, with a `role`."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ROLE_FIELD_NUMBER: builtins.int
    MEMBERS_FIELD_NUMBER: builtins.int
    CONDITION_FIELD_NUMBER: builtins.int
    role: builtins.str
    """Role that is assigned to the list of `members`, or principals.
    For example, `roles/viewer`, `roles/editor`, or `roles/owner`.
    """
    @property
    def members(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Specifies the principals requesting access for a Cloud Platform resource.
        `members` can have the following values:

        * `allUsers`: A special identifier that represents anyone who is
           on the internet; with or without a Google account.

        * `allAuthenticatedUsers`: A special identifier that represents anyone
           who is authenticated with a Google account or a service account.

        * `user:{emailid}`: An email address that represents a specific Google
           account. For example, `alice@example.com` .


        * `serviceAccount:{emailid}`: An email address that represents a service
           account. For example, `my-other-app@appspot.gserviceaccount.com`.

        * `group:{emailid}`: An email address that represents a Google group.
           For example, `admins@example.com`.

        * `deleted:user:{emailid}?uid={uniqueid}`: An email address (plus unique
           identifier) representing a user that has been recently deleted. For
           example, `alice@example.com?uid=123456789012345678901`. If the user is
           recovered, this value reverts to `user:{emailid}` and the recovered user
           retains the role in the binding.

        * `deleted:serviceAccount:{emailid}?uid={uniqueid}`: An email address (plus
           unique identifier) representing a service account that has been recently
           deleted. For example,
           `my-other-app@appspot.gserviceaccount.com?uid=123456789012345678901`.
           If the service account is undeleted, this value reverts to
           `serviceAccount:{emailid}` and the undeleted service account retains the
           role in the binding.

        * `deleted:group:{emailid}?uid={uniqueid}`: An email address (plus unique
           identifier) representing a Google group that has been recently
           deleted. For example, `admins@example.com?uid=123456789012345678901`. If
           the group is recovered, this value reverts to `group:{emailid}` and the
           recovered group retains the role in the binding.


        * `domain:{domain}`: The G Suite domain (primary) that represents all the
           users of that domain. For example, `google.com` or `example.com`.
        """

    @property
    def condition(self) -> google.type.expr_pb2.Expr:
        """The condition that is associated with this binding.

        If the condition evaluates to `true`, then this binding applies to the
        current request.

        If the condition evaluates to `false`, then this binding does not apply to
        the current request. However, a different role binding might grant the same
        role to one or more of the principals in this binding.

        To learn which resources support conditions in their IAM policies, see the
        [IAM
        documentation](https://cloud.google.com/iam/help/conditions/resource-policies).
        """

    def __init__(
        self,
        *,
        role: builtins.str = ...,
        members: collections.abc.Iterable[builtins.str] | None = ...,
        condition: google.type.expr_pb2.Expr | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["condition", b"condition"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["condition", b"condition", "members", b"members", "role", b"role"]) -> None: ...

global___Binding = Binding

@typing.final
class AuditConfig(google.protobuf.message.Message):
    """Specifies the audit configuration for a service.
    The configuration determines which permission types are logged, and what
    identities, if any, are exempted from logging.
    An AuditConfig must have one or more AuditLogConfigs.

    If there are AuditConfigs for both `allServices` and a specific service,
    the union of the two AuditConfigs is used for that service: the log_types
    specified in each AuditConfig are enabled, and the exempted_members in each
    AuditLogConfig are exempted.

    Example Policy with multiple AuditConfigs:

        {
          "audit_configs": [
            {
              "service": "allServices",
              "audit_log_configs": [
                {
                  "log_type": "DATA_READ",
                  "exempted_members": [
                    "user:jose@example.com"
                  ]
                },
                {
                  "log_type": "DATA_WRITE"
                },
                {
                  "log_type": "ADMIN_READ"
                }
              ]
            },
            {
              "service": "sampleservice.googleapis.com",
              "audit_log_configs": [
                {
                  "log_type": "DATA_READ"
                },
                {
                  "log_type": "DATA_WRITE",
                  "exempted_members": [
                    "user:aliya@example.com"
                  ]
                }
              ]
            }
          ]
        }

    For sampleservice, this policy enables DATA_READ, DATA_WRITE and ADMIN_READ
    logging. It also exempts jose@example.com from DATA_READ logging, and
    aliya@example.com from DATA_WRITE logging.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SERVICE_FIELD_NUMBER: builtins.int
    AUDIT_LOG_CONFIGS_FIELD_NUMBER: builtins.int
    service: builtins.str
    """Specifies a service that will be enabled for audit logging.
    For example, `storage.googleapis.com`, `cloudsql.googleapis.com`.
    `allServices` is a special value that covers all services.
    """
    @property
    def audit_log_configs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___AuditLogConfig]:
        """The configuration for logging of each type of permission."""

    def __init__(
        self,
        *,
        service: builtins.str = ...,
        audit_log_configs: collections.abc.Iterable[global___AuditLogConfig] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["audit_log_configs", b"audit_log_configs", "service", b"service"]) -> None: ...

global___AuditConfig = AuditConfig

@typing.final
class AuditLogConfig(google.protobuf.message.Message):
    """Provides the configuration for logging a type of permissions.
    Example:

        {
          "audit_log_configs": [
            {
              "log_type": "DATA_READ",
              "exempted_members": [
                "user:jose@example.com"
              ]
            },
            {
              "log_type": "DATA_WRITE"
            }
          ]
        }

    This enables 'DATA_READ' and 'DATA_WRITE' logging, while exempting
    jose@example.com from DATA_READ logging.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _LogType:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _LogTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[AuditLogConfig._LogType.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        LOG_TYPE_UNSPECIFIED: AuditLogConfig._LogType.ValueType  # 0
        """Default case. Should never be this."""
        ADMIN_READ: AuditLogConfig._LogType.ValueType  # 1
        """Admin reads. Example: CloudIAM getIamPolicy"""
        DATA_WRITE: AuditLogConfig._LogType.ValueType  # 2
        """Data writes. Example: CloudSQL Users create"""
        DATA_READ: AuditLogConfig._LogType.ValueType  # 3
        """Data reads. Example: CloudSQL Users list"""

    class LogType(_LogType, metaclass=_LogTypeEnumTypeWrapper):
        """The list of valid permission types for which logging can be configured.
        Admin writes are always logged, and are not configurable.
        """

    LOG_TYPE_UNSPECIFIED: AuditLogConfig.LogType.ValueType  # 0
    """Default case. Should never be this."""
    ADMIN_READ: AuditLogConfig.LogType.ValueType  # 1
    """Admin reads. Example: CloudIAM getIamPolicy"""
    DATA_WRITE: AuditLogConfig.LogType.ValueType  # 2
    """Data writes. Example: CloudSQL Users create"""
    DATA_READ: AuditLogConfig.LogType.ValueType  # 3
    """Data reads. Example: CloudSQL Users list"""

    LOG_TYPE_FIELD_NUMBER: builtins.int
    EXEMPTED_MEMBERS_FIELD_NUMBER: builtins.int
    log_type: global___AuditLogConfig.LogType.ValueType
    """The log type that this config enables."""
    @property
    def exempted_members(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Specifies the identities that do not cause logging for this type of
        permission.
        Follows the same format of [Binding.members][google.iam.v1.Binding.members].
        """

    def __init__(
        self,
        *,
        log_type: global___AuditLogConfig.LogType.ValueType = ...,
        exempted_members: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["exempted_members", b"exempted_members", "log_type", b"log_type"]) -> None: ...

global___AuditLogConfig = AuditLogConfig

@typing.final
class PolicyDelta(google.protobuf.message.Message):
    """The difference delta between two policies."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BINDING_DELTAS_FIELD_NUMBER: builtins.int
    AUDIT_CONFIG_DELTAS_FIELD_NUMBER: builtins.int
    @property
    def binding_deltas(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___BindingDelta]:
        """The delta for Bindings between two policies."""

    @property
    def audit_config_deltas(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___AuditConfigDelta]:
        """The delta for AuditConfigs between two policies."""

    def __init__(
        self,
        *,
        binding_deltas: collections.abc.Iterable[global___BindingDelta] | None = ...,
        audit_config_deltas: collections.abc.Iterable[global___AuditConfigDelta] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["audit_config_deltas", b"audit_config_deltas", "binding_deltas", b"binding_deltas"]) -> None: ...

global___PolicyDelta = PolicyDelta

@typing.final
class BindingDelta(google.protobuf.message.Message):
    """One delta entry for Binding. Each individual change (only one member in each
    entry) to a binding will be a separate entry.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Action:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _ActionEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[BindingDelta._Action.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        ACTION_UNSPECIFIED: BindingDelta._Action.ValueType  # 0
        """Unspecified."""
        ADD: BindingDelta._Action.ValueType  # 1
        """Addition of a Binding."""
        REMOVE: BindingDelta._Action.ValueType  # 2
        """Removal of a Binding."""

    class Action(_Action, metaclass=_ActionEnumTypeWrapper):
        """The type of action performed on a Binding in a policy."""

    ACTION_UNSPECIFIED: BindingDelta.Action.ValueType  # 0
    """Unspecified."""
    ADD: BindingDelta.Action.ValueType  # 1
    """Addition of a Binding."""
    REMOVE: BindingDelta.Action.ValueType  # 2
    """Removal of a Binding."""

    ACTION_FIELD_NUMBER: builtins.int
    ROLE_FIELD_NUMBER: builtins.int
    MEMBER_FIELD_NUMBER: builtins.int
    CONDITION_FIELD_NUMBER: builtins.int
    action: global___BindingDelta.Action.ValueType
    """The action that was performed on a Binding.
    Required
    """
    role: builtins.str
    """Role that is assigned to `members`.
    For example, `roles/viewer`, `roles/editor`, or `roles/owner`.
    Required
    """
    member: builtins.str
    """A single identity requesting access for a Cloud Platform resource.
    Follows the same format of Binding.members.
    Required
    """
    @property
    def condition(self) -> google.type.expr_pb2.Expr:
        """The condition that is associated with this binding."""

    def __init__(
        self,
        *,
        action: global___BindingDelta.Action.ValueType = ...,
        role: builtins.str = ...,
        member: builtins.str = ...,
        condition: google.type.expr_pb2.Expr | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["condition", b"condition"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["action", b"action", "condition", b"condition", "member", b"member", "role", b"role"]) -> None: ...

global___BindingDelta = BindingDelta

@typing.final
class AuditConfigDelta(google.protobuf.message.Message):
    """One delta entry for AuditConfig. Each individual change (only one
    exempted_member in each entry) to a AuditConfig will be a separate entry.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Action:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _ActionEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[AuditConfigDelta._Action.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        ACTION_UNSPECIFIED: AuditConfigDelta._Action.ValueType  # 0
        """Unspecified."""
        ADD: AuditConfigDelta._Action.ValueType  # 1
        """Addition of an audit configuration."""
        REMOVE: AuditConfigDelta._Action.ValueType  # 2
        """Removal of an audit configuration."""

    class Action(_Action, metaclass=_ActionEnumTypeWrapper):
        """The type of action performed on an audit configuration in a policy."""

    ACTION_UNSPECIFIED: AuditConfigDelta.Action.ValueType  # 0
    """Unspecified."""
    ADD: AuditConfigDelta.Action.ValueType  # 1
    """Addition of an audit configuration."""
    REMOVE: AuditConfigDelta.Action.ValueType  # 2
    """Removal of an audit configuration."""

    ACTION_FIELD_NUMBER: builtins.int
    SERVICE_FIELD_NUMBER: builtins.int
    EXEMPTED_MEMBER_FIELD_NUMBER: builtins.int
    LOG_TYPE_FIELD_NUMBER: builtins.int
    action: global___AuditConfigDelta.Action.ValueType
    """The action that was performed on an audit configuration in a policy.
    Required
    """
    service: builtins.str
    """Specifies a service that was configured for Cloud Audit Logging.
    For example, `storage.googleapis.com`, `cloudsql.googleapis.com`.
    `allServices` is a special value that covers all services.
    Required
    """
    exempted_member: builtins.str
    """A single identity that is exempted from "data access" audit
    logging for the `service` specified above.
    Follows the same format of Binding.members.
    """
    log_type: builtins.str
    """Specifies the log_type that was be enabled. ADMIN_ACTIVITY is always
    enabled, and cannot be configured.
    Required
    """
    def __init__(
        self,
        *,
        action: global___AuditConfigDelta.Action.ValueType = ...,
        service: builtins.str = ...,
        exempted_member: builtins.str = ...,
        log_type: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["action", b"action", "exempted_member", b"exempted_member", "log_type", b"log_type", "service", b"service"]) -> None: ...

global___AuditConfigDelta = AuditConfigDelta
